@@ -2157,6 +2157,9 @@ pango_attr_list_change (PangoAttrList  *list,
       if (tmp_attr->klass->type != attr->klass->type)
         continue;
 
+      if (tmp_attr == attr)
+        continue;
+
       if (tmp_attr->end_index <= attr->end_index ||
           pango_attribute_equal (tmp_attr, attr))
         {
@@ -2184,6 +2187,9 @@ pango_attr_list_change (PangoAttrList  *list,
 
               if (tmp_attr2->start_index >= tmp_attr->start_index)
                 break;
+
+              g_ptr_array_index (list->attributes, k - 1) = tmp_attr2;
+              g_ptr_array_index (list->attributes, k) = tmp_attr;
             }
         }
     }
@@ -2289,10 +2295,16 @@ pango_attr_list_update (PangoAttrList *list,
  * that applies at position @pos in @list by an amount @len,
  * and then calling [method@Pango.AttrList.change] with a copy
  * of each attribute in @other in sequence (offset in position
- * by @pos).
+ * by @pos, and limited in length to @len).
  *
  * This operation proves useful for, for instance, inserting
  * a pre-edit string in the middle of an edit buffer.
+ *
+ * For backwards compatibility, the function behaves differently
+ * when @len is 0. In this case, the attributes from @other are
+ * not imited to @len, and are just overlayed on top of @list.
+ *
+ * This mode is useful for merging two lists of attributes together.
  */
 void
 pango_attr_list_splice (PangoAttrList *list,
@@ -2347,8 +2359,16 @@ pango_attr_list_splice (PangoAttrList *list,
   for (i = 0, p = other->attributes->len; i < p; i++)
     {
       PangoAttribute *attr = pango_attribute_copy (g_ptr_array_index (other->attributes, i));
-      attr->start_index = MIN (CLAMP_ADD (attr->start_index, upos), end);
-      attr->end_index = MIN (CLAMP_ADD (attr->end_index, upos), end);
+      if (ulen > 0)
+        {
+          attr->start_index = MIN (CLAMP_ADD (attr->start_index, upos), end);
+          attr->end_index = MIN (CLAMP_ADD (attr->end_index, upos), end);
+        }
+      else
+        {
+          attr->start_index = CLAMP_ADD (attr->start_index, upos);
+          attr->end_index = CLAMP_ADD (attr->end_index, upos);
+        }
 
       /* Same as above, the attribute could be squashed to zero-length; here
        * pango_attr_list_change() will take care of deleting it.
@@ -2624,7 +2644,11 @@ attr_print (GString        *str,
            attr->klass->type == PANGO_ATTR_FALLBACK)
     g_string_append (str, ((PangoAttrInt *)attr)->value ? " true" : " false");
   else if ((string = pango_attribute_as_string (attr)) != NULL)
-    g_string_append_printf (str, " %s", string->value);
+    {
+      char *s = g_strescape (string->value, NULL);
+      g_string_append_printf (str, " \"%s\"", s);
+      g_free (s);
+    }
   else if ((lang = pango_attribute_as_language (attr)) != NULL)
     g_string_append_printf (str, " %s", pango_language_to_string (lang->value));
   else if ((integer = pango_attribute_as_int (attr)) != NULL)
@@ -2632,13 +2656,15 @@ attr_print (GString        *str,
   else if ((flt = pango_attribute_as_float (attr)) != NULL)
     {
       char buf[20];
-      g_ascii_formatd (buf, 20, " %f", flt->value);
+      g_ascii_formatd (buf, 20, "%f", flt->value);
       g_string_append_printf (str, " %s", buf);
     }
   else if ((font = pango_attribute_as_font_desc (attr)) != NULL)
     {
       char *s = pango_font_description_to_string (font->desc);
-      g_string_append_printf (str, " \"%s\"", s);
+      char *s2 = g_strescape (s, NULL);
+      g_string_append_printf (str, " \"%s\"", s2);
+      g_free (s2);
       g_free (s);
     }
   else if ((color = pango_attribute_as_color (attr)) != NULL)
@@ -2663,12 +2689,38 @@ attr_print (GString        *str,
  *
  * Serializes a `PangoAttrList` to a string.
  *
- * No guarantees are made about the format of the string,
- * it may change between Pango versions.
+ * In the resulting string, serialized attributes are separated by newlines or commas.
+ * Individual attributes are serialized to a string of the form
  *
- * The intended use of this function is testing and
- * debugging. The format is not meant as a permanent
- * storage format.
+ *   START END TYPE VALUE
+ *
+ * Where START and END are the indices (with -1 being accepted in place
+ * of MAXUINT), TYPE is the nickname of the attribute value type, e.g.
+ * _weight_ or _stretch_, and the value is serialized according to its type:
+ *
+ * - enum values as nick or numeric value
+ * - boolean values as _true_ or _false_
+ * - integers and floats as numbers
+ * - strings as string, optionally quoted
+ * - font features as quoted string
+ * - PangoLanguage as string
+ * - PangoFontDescription as serialized by [method@Pango.FontDescription.to_string], quoted
+ * - PangoColor as serialized by [method@Pango.Color.to_string]
+ *
+ * Examples:
+ *
+ * ```
+ * 0 10 foreground red, 5 15 weight bold, 0 200 font-desc "Sans 10"
+ * ```
+ *
+ * ```
+ * 0 -1 weight 700
+ * 0 100 family Times
+ * ```
+ *
+ * To parse the returned value, use [func@Pango.AttrList.from_string].
+ *
+ * Note that shape attributes can not be serialized.
  *
  * Returns: (transfer full): a newly allocated string
  * Since: 1.50
@@ -2787,6 +2839,7 @@ pango_attr_list_from_string (const char *text)
       PangoFontDescription *desc;
       PangoColor color;
       double num;
+      int len;
 
       start_index = g_ascii_strtoll (p, &endp, 10);
       if (*endp != ' ')
@@ -2832,7 +2885,10 @@ pango_attr_list_from_string (const char *text)
 
 #define ENUM_ATTR(name, type, min, max) \
           endp = (char *)p + strcspn (p, ",\n"); \
-          integer = get_attr_value (attr_type, p, endp - p); \
+          len = endp - p; \
+          while (len > 0 && p[len - 1] == ' ') \
+            len--; \
+          integer = get_attr_value (attr_type, p, len); \
           attr = pango_attr_##name##_new ((type) CLAMP (integer, min, max));
 
 #define FLOAT_ATTR(name) \
@@ -2870,7 +2926,22 @@ pango_attr_list_from_string (const char *text)
         case PANGO_ATTR_FAMILY:
           endp = (char *)p + strcspn (p, ",\n");
           if (!is_valid_end_char (*endp)) goto fail;
-          str = g_strndup (p, endp - p);
+          if (p[0] == '"')
+            {
+              char *str2;
+
+              len = endp - p;
+              while (len > 0 && p[len - 1] == ' ')
+                len--;
+
+              if (p[len - 1] != '"') goto fail;
+
+              str2 = g_strndup (p + 1, len - 2);
+              str = g_strcompress (str2);
+              g_free (str2);
+            }
+          else
+            str = g_strndup (p, endp - p);
           attr = pango_attr_family_new (str);
           g_free (str);
           break;
@@ -2896,6 +2967,7 @@ pango_attr_list_from_string (const char *text)
           break;
 
         case PANGO_ATTR_FONT_DESC:
+          if (*p != '"') goto fail;
           p++;
           endp = strchr (p, '"');
           if (!endp) goto fail;
