@@ -66,13 +66,15 @@
  *
  * Looks up the script for a particular character.
  *
- * The script of a character is defined by Unicode Standard Annex \#24.
+ * The script of a character is defined by
+ * [Unicode Standard Annex 24: Script names](http://www.unicode.org/reports/tr24/).
+ *
  * No check is made for @ch being a valid Unicode character; if you pass
  * in invalid character, the result is undefined.
  *
  * Note that while the return type of this function is declared
  * as `PangoScript`, as of Pango 1.18, this function simply returns
- * the return value of g_unichar_get_script(). Callers must be
+ * the return value of [func@GLib.unichar_get_script]. Callers must be
  * prepared to handle unknown values.
  *
  * Return value: the `PangoScript` for the character.
@@ -120,7 +122,7 @@ _pango_script_iter_init (PangoScriptIter *iter,
 /**
  * pango_script_iter_new:
  * @text: a UTF-8 string
- * @length: length of @text, or -1 if @text is nul-terminated.
+ * @length: length of @text, or -1 if @text is nul-terminated
  *
  * Create a new `PangoScriptIter`, used to break a string of
  * Unicode text into runs by Unicode script.
@@ -177,12 +179,13 @@ pango_script_iter_free (PangoScriptIter *iter)
  * @script: (out) (optional): location to store script for range
  *
  * Gets information about the range to which @iter currently points.
+ *
  * The range is the set of locations p where *start <= p < *end.
  * (That is, it doesn't include the character stored at *end)
  *
  * Note that while the type of the @script argument is declared
  * as `PangoScript`, as of Pango 1.18, this function simply returns
- * GUnicodeScript values. Callers must be prepared to handle unknown
+ * `GUnicodeScript` values. Callers must be prepared to handle unknown
  * values.
  *
  * Since: 1.4
@@ -283,12 +286,29 @@ get_pair_index (gunichar ch)
   return -1;
 }
 
+static gboolean
+is_cluster_extender (gunichar ch)
+{
+  GUnicodeType type = g_unichar_type (ch);
+  return (type >= G_UNICODE_SPACING_MARK && type <= G_UNICODE_NON_SPACING_MARK) ||
+	 (ch >= 0x200C && ch <= 0x200D) ||    /* ZWJ, ZWNJ */
+	 (ch >= 0xFF9E && ch <= 0xFF9F) ||    /* katakana sound marks */
+	 (ch >= 0x1F3FB && ch <= 0x1F3FF) ||  /* fitzpatrick skin tone modifiers */
+	 (ch >= 0xE0020 && ch <= 0xE007F);    /* emoji (flag) tag characters */
+}
+
 /* duplicated in pango-language.c */
 #define REAL_SCRIPT(script) \
   ((script) > PANGO_SCRIPT_INHERITED && (script) != PANGO_SCRIPT_UNKNOWN)
 
-#define SAME_SCRIPT(script1, script2) \
-  (!REAL_SCRIPT (script1) || !REAL_SCRIPT (script2) || (script1) == (script2))
+#define IS_CLUSTER_EXTENDER(ch) \
+  g_unichar_type (ch)
+
+/* TODO: Use Unicode ScriptExtensions */
+#define SAME_SCRIPT(script1, script2, ch) \
+  (!REAL_SCRIPT (script1) || !REAL_SCRIPT (script2) || \
+   (script1) == (script2) || \
+   is_cluster_extender (ch))
 
 #define IS_OPEN(pair_index) (((pair_index) & 1) == 0)
 
@@ -369,7 +389,7 @@ pango_script_iter_next (PangoScriptIter *iter)
 	    }
 	}
 
-      if (SAME_SCRIPT (iter->script_code, sc))
+      if (SAME_SCRIPT (iter->script_code, sc, ch))
 	{
 	  if (!REAL_SCRIPT (iter->script_code) && REAL_SCRIPT (sc))
 	    {
