@@ -282,6 +282,16 @@
  */
 #define PANGO_VERSION_1_50       (G_ENCODE_VERSION (1, 50))
 
+/**
+ * PANGO_VERSION_1_52:
+ *
+ * A macro that evaluates to the 1.52 version of Pango, in a format
+ * that can be used by the C pre-processor.
+ *
+ * Since: 1.52
+ */
+#define PANGO_VERSION_1_52       (G_ENCODE_VERSION (1, 52))
+
 /* evaluates to the current stable version; for development cycles,
  * this means the next stable target
  */
@@ -342,7 +352,7 @@
  * functions, then using functions added after version
  * %PANGO_VERSION_MAX_ALLOWED will cause warnings.
  *
- * Unless you are using PANGO_CHECK_VERSION() or the like to compile
+ * Unless you are using PANGO_VERSION_CHECK() or the like to compile
  * different code depending on the Pango version, then this should be
  * set to the same value as %PANGO_VERSION_MIN_REQUIRED.
  *
@@ -729,4 +739,18 @@
 # define PANGO_AVAILABLE_IN_1_50                _PANGO_EXTERN
 #endif
 
+#if PANGO_VERSION_MIN_REQUIRED >= PANGO_VERSION_1_52
+# define PANGO_DEPRECATED_IN_1_52               PANGO_DEPRECATED
+# define PANGO_DEPRECATED_IN_1_52_FOR(f)        PANGO_DEPRECATED_FOR(f)
+#else
+# define PANGO_DEPRECATED_IN_1_52               _PANGO_EXTERN
+# define PANGO_DEPRECATED_IN_1_52_FOR(f)        _PANGO_EXTERN
+#endif
+
+#if PANGO_VERSION_MAX_ALLOWED < PANGO_VERSION_1_52
+# define PANGO_AVAILABLE_IN_1_52                PANGO_UNAVAILABLE(1, 52)
+#else
+# define PANGO_AVAILABLE_IN_1_52                _PANGO_EXTERN
+#endif
+
 #endif /* __PANGO_VERSION_H__ */
